import json
import time
from io import BytesIO

import oss2
import redis
from PIL import Image
from fastapi import Body, Depends, APIRouter, Query
import asyncio
import random
import re
from parsel import Selector
from pydantic import BaseModel, Field
from utils.ptools import redis_host, fetch_url
from utils.ptools import fs_send_msg, cookies, format_value, clean_html, clean_title, put_logs, get_re_content, \
    generate_md5,upload_image
from loguru import logger
import aiohttp

readable_characters_pattern = re.compile(r'[\u4e00-\u9fffa-zA-Z0-9]')


headers = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
    'Accept-Language': 'zh-CN,zh;q=0.9',
    'Cache-Control': 'max-age=0',
    'Connection': 'close',
    'Sec-Fetch-Dest': 'document',
    'Sec-Fetch-Mode': 'navigate',
    'Sec-Fetch-Site': 'none',
    'Sec-Fetch-User': '?1',
    'Upgrade-Insecure-Requests': '1',
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36',
    'sec-ch-ua': '"Not/A)Brand";v="8", "Chromium";v="126", "Google Chrome";v="126"',
    'sec-ch-ua-mobile': '?0',
    'sec-ch-ua-platform': '"macOS"',
}


async def main_query(query, timeout, require_redis, require_filter):
    query_url = f"https://cn.bing.com/images/search?q={query}&go=搜索&qs=ds&form=QBIR"
    list_url = [query_url]
    tasks = [fetch_url(url, timeout, max_retries=10, cookies_bool=True, check_value="dgControl_list") for url in list_url]
    results = await asyncio.gather(*tasks)
    icon_list = []
    key_values = {}
    for index_, result in enumerate(results):
        if result:
            soup = Selector(result)
            li_list = soup.css("ul.dgControl_list li")
            for li in li_list:
                imgpt = li.css("div.iuscp div.imgpt a.iusc::attr(m)").get()
                if imgpt:
                    img_data = json.loads(imgpt)
                    img_url = img_data['turl'].strip()
                    # 特殊处理
                    if img_url == "https://tse1-mm.cn.bing.net/th?id=OIP.otKgF7eVJduviIgp6ZHGGQDIDI&pid=15.1":
                        img_url = "https://image.qcc.com/product/80d5dab5bbb58ff09fd1d5550895d6df.jpg?x-oss-process=style/logo_200"
                    desp = format_value(img_data['t'])
                    icon_list.append(img_url)
                    key_values[img_url] = desp
                    # print(img_url, desp)
                if len(icon_list) >= 2:
                    break
    redis_image_items = {}
    icon_list_yes = icon_list
    icon_list_no = []
    if require_redis:
        redis_conn = redis.Redis(host=redis_host, port=6379, db=6, socket_timeout=1)
        for url in icon_list:
            key_value = redis_conn.get(url)
            if key_value:
                new_item = json.loads(key_value)
                redis_image_items[url] = new_item
                icon_list_no.append(url)
        redis_conn.close()
    icon_list_con = [x for x in icon_list_yes if x not in icon_list_no]
    if len(icon_list_con) > 0:
        no_redis_image_items, error_count = await upload_image(icon_list_con, timeout)
        image_items = {**redis_image_items, **no_redis_image_items}
    else:
        image_items = redis_image_items

    for one in list(image_items.keys()):
        image_items[one]['desp'] = key_values[one]

    if require_redis:
        redis_conn = redis.Redis(host=redis_host, port=6379, db=6, socket_timeout=1)
        for url in image_items:
            url_values = image_items[url]
            redis_conn.setex(url, 60*60*24, json.dumps(url_values, ensure_ascii=False))

    error_count = 0
    for one in list(image_items.keys()):
        print(image_items[one]['oss_url'])
        if not image_items[one]['oss_url']:
            error_count += 1
            del image_items[one]

    # if require_filter:
    #     for one in list(image_items.keys()):
    #         one_value = image_items[one]
    #         width = one_value['width']
    #         height = one_value['height']
    #         if width and height:
    #             if width > 150 and height > 150 and 0.5 < (width / height) < 2.5:
    #                 pass
    #             else:
    #                 del image_items[one]
    #         else:
    #             error_count += 1
    #             del image_items[one]

    logger.info(f'info== icon {len(icon_list)} 张，下载完成，返回数据 {len(image_items)} 条')
    logger.info(f'info==3 成功获取item_list 共 {len(image_items)} 条数据')
    return image_items, error_count


# 定义路由
bingImageQuery_router = APIRouter()


class FindGDListRequest(BaseModel):
    timeout: int = 1
    require_redis: bool = True
    require_filter: bool = False
    query: str = Query(..., title="关键字"),


@bingImageQuery_router.post(
    "/chain/aiSearch/bingImageQueryRedis",
    summary="图片查询下载",
    response_description='''
    {
        "timeout": 1, 
        "require_redis": true,
        "require_filter": false,
        "query": 浙江万维空间信息技术有限公司
    }
    '''
)
async def find_sg_selective(
        request: FindGDListRequest = Body(None, title="传入参数")
):
    '''
    bing 提交query获取图片相关信息以及下载地址(post)
    '''
    start_date = time.time()
    require_redis = request.require_redis
    require_filter = request.require_filter
    timeout = request.timeout
    query = request.query
    try:
        sq_list, error_count = await main_query(query, timeout, require_redis, require_filter)
        end_date = time.time()
        data = {"status": 200,
                "message": "查询成功",
                'time': "{:.2f}".format(end_date - start_date),
                'warn_count': error_count, 'data': sq_list}
        contents = [
            ('log_type', "success"),
            ('time', ''),
            ('query', query),
            ('api_type', 'bing_images_query'),
            ('msg', ""),
            ('remark', 'bing_images_query 获取网页成功'),
            ('response', json.dumps(data, ensure_ascii=False)),
        ]
        put_logs(contents)
    except Exception as e:
        end_date = time.time()
        data = {"status": 0,
                "message": "查询失败，请联系管理员",
                'time': "{:.2f}".format(end_date - start_date),
                'warn_count': 0, 'data': []}
        fs_send_msg("error", "bing_images_query", f"搜索timeout={timeout}, query={query}  异常报错 {e}")
        contents = [
            ('log_type', "error"),
            ('time', ''),
            ('query', query),
            ('api_type', 'bing_images_query'),
            ('msg', f"搜索timeout={timeout}, query={query}  异常报错 {e}"),
            ('remark', 'bing_images_query 获取网页失败'),
            ('response', json.dumps(data, ensure_ascii=False)),
        ]
        put_logs(contents)
    return data
