import json
import re
import time
from fastapi import Body, Depends, APIRouter, Query
import asyncio
import hashlib
from utils.ptools import fs_send_msg, get_cookies, get_domain, clean_title, put_logs, false_url,oss_key, oss_value
import oss2
from datetime import datetime
from loguru import logger
import aiohttp
from parsel import Selector


headers = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
    'Accept-Language': 'zh-CN,zh;q=0.9',
    'Cache-Control': 'max-age=0',
    'Connection': 'close',
    'Sec-Fetch-Dest': 'document',
    'Sec-Fetch-Mode': 'navigate',
    'Sec-Fetch-Site': 'none',
    'Sec-Fetch-User': '?1',
    'Upgrade-Insecure-Requests': '1',
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36',
    'sec-ch-ua': '"Not/A)Brand";v="8", "Chromium";v="126", "Google Chrome";v="126"',
    'sec-ch-ua-mobile': '?0',
    'sec-ch-ua-platform': '"macOS"',
}


def generate_md5(string):  # 字符串转md5
    m = hashlib.md5()
    m.update(string.encode('utf-8'))
    return m.hexdigest()



async def download_image(url):
    image_timeout = 2
    image_data = None
    if not url:
        return url, image_data
    if url.startswith("http"):
        retries = 0
        try:
            conn = aiohttp.TCPConnector(ssl=False)
            async_timeout = aiohttp.ClientTimeout(total=image_timeout)
            async with aiohttp.ClientSession(connector=conn) as session:
                async with session.get(url, timeout=async_timeout) as response:
                    image_data = await response.read()
                    return url, image_data
        except asyncio.TimeoutError as e:
            retries += 1
            logger.error(f'error==图片 请求超时错误 timeout={image_timeout}：Retry {retries} for {url} -- {e}')
        except aiohttp.ClientError as e:
            retries += 1
            logger.error(f'error==图片 客户端请求错误 timeout={image_timeout}：Retry {retries} for {url} -- {e}')
        except Exception as e:
            retries += 1
            logger.error(f'error==图片 请求未知错误 timeout={image_timeout}：Retry {retries} for {url} -- {e}')
        finally:
            return url, image_data
    else:
        return url, image_data


async def upload_image(url_list):
    img_item = {}
    tasks = [download_image(url) for url in url_list]
    image_datas = await asyncio.gather(*tasks)
    for url, image_data in image_datas:
        if url:
            if image_data:
                file_name = generate_md5(url) + ".png"
                # 这个需要自己配置oss地址
                auth = oss2.Auth(oss_key, oss_value)
                bucket = oss2.Bucket(auth, 'oss-cn-hangzhou-internal.aliyuncs.com', 'lianqiai-public')
                result = bucket.put_object('upload/bing_logo/' + file_name, image_data)
                if result.status == 200:
                    oss_url = f'https://lianqiai-public.oss-cn-hangzhou.aliyuncs.com/upload/bing_logo/{file_name}'
                    # print(f"上传成功 url长度{len(url)}-- {url} -- {oss_url}")
                    img_item[generate_md5(url)] = {
                        'url': url,
                        'oss_url': oss_url
                    }
                else:
                    logger.error(f"文件上传失败 {result.status} {result.resp.body}")
            else:
                img_item[generate_md5(url)] = {
                    'url': url,
                    'oss_url': None
                }
    return img_item


def time_to_int(date_str2):
    date_str1 = '1970-01-01'
    date1 = datetime.strptime(date_str1, '%Y-%m-%d')
    date2 = datetime.strptime(date_str2, '%Y-%m-%d')
    delta = date2 - date1
    days_difference = delta.days
    return days_difference


async def fetch_url(url, timeout, max_retries=1, cookies_bool=False, check_value=None):
    retries = 0
    while retries < max_retries:
        try:
            conn = aiohttp.TCPConnector(ssl=False)
            async_timeout = aiohttp.ClientTimeout(total=timeout)
            async with aiohttp.ClientSession(headers=headers, connector=conn) as session:
                if cookies_bool:
                    cookies = get_cookies()
                    async with session.get(url, timeout=async_timeout, cookies=cookies) as response:
                        rex_text = await response.text()
                        if check_value and check_value not in rex_text:
                            raise Exception(f"check_value 不存在== {check_value}")
                        return rex_text
                else:
                    async with session.get(url, timeout=async_timeout) as response:
                        rex_text = await response.text()
                        if check_value and check_value not in rex_text:
                            raise Exception(f"check_value 不存在== {check_value}")
                        return rex_text
        except asyncio.TimeoutError as e:
            retries += 1
            logger.error(f'error== 请求超时错误 timeout={timeout}：Retry {retries} for {url} -- {e}')
        except aiohttp.ClientError as e:
            retries += 1
            logger.error(f'error== 客户端请求错误 timeout={timeout}：Retry {retries} for {url} -- {e}')
        except Exception as e:
            retries += 1
            logger.error(f'error== 请求未知错误 timeout={timeout}：Retry {retries} for {url} -- {e}')
    logger.error(f"error==  请求重试后最终失败{url}")
    return None


async def main_query(query, max_page, freshness, require_icon):
    old_max_page = max_page  # 如果需要数量够，可以max_page+n
    max_page += 1
    if freshness and ".." in freshness:
        start_date, end_date = freshness.split("..")
    else:
        start_date, end_date = None, None
    logger.info(
        f'info==1 开始查询query={query} max_page={max_page} start_date={start_date} end_date={end_date} require_icon={require_icon}')
    list_url = []
    for page in range(0, old_max_page):
        first_ = 1 if page == 0 else page * 10
        if start_date and end_date:
            ex_start_date = time_to_int(start_date)
            ex_end_date = time_to_int(end_date)
            url = f'https://cn.bing.com/search?q={query}&form=QBLH&first={first_}&FORM=PERE&&filters=ex1:"ez5_{ex_start_date}_{ex_end_date}"'
        else:
            url = f'https://cn.bing.com/search?q={query}&form=QBLH&first={first_}&FORM=PERE'
        list_url.append(url)
    logger.info(f'info==2 初步获取列表url_list {list_url}')
    tasks = [fetch_url(url, 3, max_retries=10, cookies_bool=True, check_value="b_algoSlug") for url in list_url]
    results = await asyncio.gather(*tasks)
    detail_urls = []
    item_list = []
    icon_list = []
    for index_, result in enumerate(results):
        if result:
            soup = Selector(result)
            b_algo_li_list = soup.css('li.b_algo')
            if len(b_algo_li_list) == 0:
                logger.info(f"b_algo_li_list 解析为空 = {result}")
            else:
                logger.info(f"b_algo_li_list 条数 = {len(b_algo_li_list)}")

            for b_algo_li in b_algo_li_list:
                icon_str = b_algo_li.css('div.rms_iac[data-class="rms_img"]::attr(data-src)').get()
                if icon_str:
                    icon_str = re.sub(r'&w=32&h=32&qlt.*', '', icon_str)
                b_lineclamp4 = b_algo_li.css('p.b_algoSlug *::text').getall()
                if len(b_lineclamp4) > 0:
                    date = b_lineclamp4[1].replace("\u2002", "").split('·')[0] if "·" in b_lineclamp4[1] else ""
                    date = date.replace(" ", "")
                    if not date:
                        date = None
                    snippet = "".join(b_lineclamp4[1:]).split(' · ')[1] if " · " in b_lineclamp4[1] else "".join(
                        b_lineclamp4[1:])
                    link = b_algo_li.css('h2 a::attr(href)').get()
                    if link:
                        link = link.replace('baike.baidu.hk', 'baike.baidu.com')
                    publish = b_algo_li.css('div.tptt::text').get()
                else:
                    publish = b_algo_li.css('div.tptt::text').get()
                    link = b_algo_li.css('h2 a::attr(href)').get()
                    snippet = "".join(b_algo_li.css('div.b_caption *::text').getall()).replace("\u2002", "")
                    date = ""
                    icon_str = b_algo_li.css('div.rms_iac[data-class="rms_img"]::attr(data-src)').get()
                    if icon_str:
                        icon_str = re.sub(r'&w=32&h=32&qlt.*', '', icon_str)

                retrieval_dict = {
                    "publish": publish,
                    "title": clean_title("".join(b_algo_li.css('h2 *::text').getall())),
                    "link": link,
                    "snippet": snippet,
                    "ext_attrs": {
                        "date": date
                    },
                    "icon": {
                        "url": icon_str,
                        "oss_url": None,
                    }
                }
                if not snippet:
                    logger.error(f"error==snippet  b_lineclamp4 解析为空 = {b_algo_li.get()}")

                if not link:
                    logger.error(f"error==link  b_lineclamp4 解析为空 = {b_algo_li.get()}")

                if any(domin in link for domin in false_url):
                    logger.info(f"规则筛选触发 {link}")
                elif link and snippet:
                    icon_list.append(icon_str)
                    detail_urls.append(link)
                    item_list.append(retrieval_dict)
                if len(item_list) >= 20:
                    break
        else:
            logger.error(f'error== 列表url数据请求为空 {list_url[index_]}')
        if len(item_list) >= 20:
            break

    item_list = item_list[:10 * old_max_page]
    logger.info(f'info==3 成功获取item_list 共 {len(item_list)} 条数据')
    if not require_icon:
        return item_list, 0
    image_items = await upload_image(icon_list)
    image_bool = False
    for item in item_list:
        if item['icon'].get('url', ""):
            if "5595-40BA-9F13" in item['icon']['url']:
                item['icon']['oss_url'] = None
            else:
                image_bool = True
                key_ = generate_md5(item['icon']['url'])
                item['icon'] = image_items[key_]
        else:
            item['icon']['oss_url'] = None
    if not image_bool:
        fs_send_msg("error", "bing列表搜索", f"cookie失效 图片异常")
    logger.info(f'info== icon {len(icon_list)} 张，下载完成，返回数据 {len(item_list)} 条')
    return item_list, 0


# 定义路由
bingList_router = APIRouter()


@bingList_router.get(
    "/chain/aiSearch/bingListQuery",
    summary="bing_list引擎搜索",
    response_description='''
    {
        "query": "万维空间",  # 关键字,
        "freshness": "2024-01-01..2024-06-01",  # 时间筛选,
        "max_pages": 2",    # 采集总页数
        "require_icon": True,  # 是否下载图片
    }
    '''
)
async def find_sg_selective(
        query: str = Query(..., title="关键字/企业"),
        freshness: str = Query("", title="时间筛选"),
        max_pages: int = Query(2, title="最大页数"),
        require_icon: bool = Query(False, title="是否下载图片"),
):
    '''
        bing 根据关键词查询搜索列表结果
    '''
    # sq_list, error_count = await main_query(query, max_pages, freshness, require_icon)
    # data = {"status": 200, "message": "查询成功", 'warn_count': error_count, 'data': sq_list}
    start_date = time.time()
    try:
        sq_list, error_count = await main_query(query, max_pages, freshness, require_icon)
        end_date = time.time()
        data = {"status": 200, "message": "查询成功",
                'time': "{:.2f}".format(end_date - start_date),
                'count': len(sq_list),
                'error_count': error_count, 'data': sq_list}
        if len(sq_list) == 0:
            log_type = "error"
            remark = "关键词查询列表失败"
            fs_send_msg("error", "bing_list",
                        f"搜索 query={query}, max_pages={max_pages}, freshness={freshness}, require_icon={require_icon} 查询返回结果为空")
        else:
            log_type = "success"
            remark = "关键词查询列表成功"
        contents = [
            ('log_type', log_type),
            ('time', ""),
            ('query', query),
            ('api_type', 'bing_list'),
            ('msg', f"搜索 query={query}, max_pages={max_pages}, freshness={freshness}, require_icon={require_icon} 成功"),
            ('remark', remark),
            ('response', json.dumps(data, ensure_ascii=False)),
        ]
        put_logs(contents)
    except Exception as e:
        end_date = time.time()
        data = {"status": 0, "message": "查询失败，请联系管理员",
                'time': "{:.2f}".format(end_date - start_date),
                'warn_count': 0,
                'data': []}
        fs_send_msg("error", "bing_list",
                    f"搜索 query={query}, max_pages={max_pages}, freshness={freshness}, require_icon={require_icon} 异常报错{e}")
        contents = [
            ('log_type', "error"),
            ('time', ""),
            ('query', query),
            ('api_type', 'bing_list'),
            ('msg',
             f"搜索 query={query}, max_pages={max_pages}, freshness={freshness}, require_icon={require_icon} 异常报错{e}"),
            ('remark', '关键词查询列表失败'),
            ('response', json.dumps(data, ensure_ascii=False)),
        ]
        put_logs(contents)
    return data
